%% Initialize
clear; close all;
load clusterdata;
figure(1);
plot(y(1,:),y(2,:),'.','Color',[0.5 0.5 1]);
axis([-8 8 -7 6])
set(gca,'FontSize',14);

% Set up a DP prior
alpha = 0.3;
% Base distribution G0 = NIW(nu0, V0)
V0 = diag([1,1,1/25]);
nu0 = 2;

% Compute some constants
[d,N] = size(y); % d = dim(y), N = #data points
dy = 1:d;
d1 = d+1;
% Vector containting marginal pdf of data points p(y_n) for n=1,...,N
py = sttpdf(y, nu0-d+1, (nu0-d+1)*V0(d1,d1)/(1+V0(d1,d1))*inv(V0(dy,dy)))';

%% Run a Gibbs sampler based on  the CRP
z = ones(1,N);
K = 1; % Number of active clusters
Klog(1) = K;
mk = N;

mklog = cell(1,1000);
mu = cell(1,1000);
Sigma = cell(1,1000);
for(r = 2:1000)
    % Sample the parameters {phi_k} | z, y  ===============================
    mu{r} = zeros(d,1,K);
    Sigma{r} = zeros(d,d,K);
    mklog{r} = mk;
    
    for(k = 1:K)
        ik = (z == k);
        nk = sum(ik);
        
        nuk = nu0 + nk;
        Dk = [y(:,ik) ; ones(1,nk)];
        Vk  = V0 + Dk*Dk';
        Sk  = Vk(dy,dy) - Vk(dy,d1)*(Vk(d1,d1)\Vk(d1,dy));
        
        Sigma{r}(:,:,k) = iwishrnd((Sk+Sk')/2, nuk);
        mu{r}(:,:,k) = mvnrnd(Vk(dy,d1)/Vk(d1,d1), Sigma{r}(:,:,k)/Vk(d1,d1));
    end
 
    % Sample new labels: z_n | z_(-n), {phi_k}, y  ========================
    q = zeros(N, K);
    % Compute p(y_n | th_k) for n = 1...N (rows) and k = 1...K (cols)
    for(k = 1:K)
        q(:,k) = mvnpdf(y', mu{r}(:,:,k)', Sigma{r}(:,:,k));
    end

    for(n = 1:N)
        mk(z(n)) = mk(z(n)) - 1;       
        if(mk(z(n)) == 0) % No other observations are associated with label z_n - remove it
            ind = [1:z(n)-1 z(n)+1:K];
            q = q(:, ind);
            mk = mk(ind);
            z(z > z(n)) = z(z > z(n)) -1; % Decrease the leading components numbers by 1
            K = K - 1;
        end
        
        % CRP probabilities
        qnow = q(n,:).*mk;
        qnew = alpha*py(n);
        qnow = [qnow qnew];
        qnow = qnow/sum(qnow);
        z(n) = DiscreteSample(qnow);
        
        if(z(n) > K) % We spawn a new component
            K = K+1;
            mk = [mk 1];
            
            % Posterior for the new cluster
            VK = V0 + [y(:,n) ; 1]*[y(:,n) ; 1]';
            SK  = VK(dy,dy) - VK(dy,d1)*(VK(d1,d1)\VK(d1,dy));
            Sigma{r} = cat(3, Sigma{r}, iwishrnd((SK+SK')/2, nu0+1));
            mu{r} = cat(3, mu{r}, mvnrnd(VK(dy,d1)/VK(d1,d1), Sigma{r}(:,:,K)/VK(d1,d1))');
            q = [q mvnpdf(y', mu{r}(:,:,K)', Sigma{r}(:,:,K))];
        else
            mk(z(n)) = mk(z(n)) + 1;
        end
    end
    Klog(r) = K;
    
    % Plot  ===============================================================
    figure(1);
    plot(y(1,:),y(2,:),'.','Color',[0.5 0.5 1]); hold on;
    for(k = 1:K)
        if(sum(z == k) > 0.01*N) % Only plot clusters with at least 1% of the data associated with them 
            plot(mu{r}(1,:,k), mu{r}(2,:,k),'k','marker','x','markersize',14,'linewidth',2);
            H = error_ellipse(Sigma{r}(:,:,k), mu{r}(:,:,k));
            set(H,'color','r','linestyle','-','linewidth',2);
        end
    end
    title(sprintf('Iteration: %i, #clusters: %i',r,K));
    axis([-8 8 -7 6])
    set(gca,'FontSize',14);
    hold off;
    drawnow;
end

%%
figure(2);
% Compute predictive pdf
[X,Y] = meshgrid(-6:0.1:6,-5:0.1:5);
Z = zeros(size(X));
% Marginal p(y)
p0 = sttpdf([X(:) Y(:)]', nu0-d+1, (nu0-d+1)*V0(d1,d1)/(1+V0(d1,d1))*inv(V0(dy,dy)))';
p0 = reshape(p0, size(X));
rvec = 500:10:1000; % Subsample Markov chain after burnin
for(r = rvec) 
    K = length(mklog{r});
    for(k = 1:K)
        tmp = mvnpdf([X(:), Y(:)],mu{r}(:,:,k)',Sigma{r}(:,:,k));
        tmp = reshape(tmp, size(X));
        Z = Z + mklog{r}(k)/(alpha+N)*tmp;
        Z = Z + alpha/(alpha+N)*p0;
    end
end
Z = Z/length(rvec);
mesh(X,Y,Z);
set(gca,'FontSize',14)
title('Posterior predictive PDF, p(y_{new} \mid y_{1:N})');

%%
figure(3);
hist(Klog(200:end),1:max(Klog(200:end)))
set(gca,'FontSize',14);
xlabel('Number of clusters','Interpreter','latex','FontSize',18)
